/*

    ConvertableData

    Class that allows access to elements of a DAL Data object 
    regardless of its type.

    (c) 1998 European Space Agency - written by Marco Beijersbergen
    
    Moved to separate utils package by H Siddiqui

*/

/* SAS */
#include <Dal.h>
#include <errstr.h>

/* C++ */
#include <fstream> 
#include <set>

/* C */
#include <cstdlib>


class ConvertableData
{

  public:

    ConvertableData() : _data(0) {}

    ConvertableData(Data* d) { data(d); }

    void data(Data* d);

    double asDouble(unsigned row);

    string operator[](unsigned row);

  private:

    Dal::DataType _type;

    void *_data;
};


// Utility function to determine whether a column is numeric or not.

static Dal::DataType numericTypesArray[] =
	{ Dal::Int8, Dal::Int16, Dal::Int32, Dal::Real32, Dal::Real64 };
static set<Dal::DataType,less<Dal::DataType> > numericTypes(
	&numericTypesArray[0],
	&numericTypesArray[sizeof(numericTypesArray)
		/sizeof(numericTypesArray[0])]);

bool isNumeric(Dal::DataType tp);
