// 
// Module: TablePixelIterator
// Author: Hassan Siddiqui
//
// Description: 
//
// Class representation of an iterator that handles DAL-based maps
// in table format.
//
// Modifications: See ChangeLog
// 
// $Id: TablePixelIterator.h,v 1.2 2000/10/06 15:24:56 hsiddiqu Exp $

#ifndef TABLEPIXELITERATOR_H
#define TABLEPIXELITERATOR_H

// General classes
#include <iostream>

// SAS-specific classes
#include "Dal.h"

// General utility functions
#include "errstr.h"
#include "Tracer.h"

// Local classes
#include "PixelIterator.h"
#include "WCSConvertor.h"
#include "PixelElement.h"

class TablePixelIterator : public PixelIterator
{
public:

  /* Constructors */
  TablePixelIterator( const string & tablename,
		      const string & xcolname,
		      const string & ycolname,
		      const string & valuecolname); 
  
  /* Destructors */
  ~TablePixelIterator();

  /* Iterator methods */
  void first ();
  void next  ();
  bool isDone();
  PixelElement currentItem();

  /* WCSConvertor object */
  const WCSConvertor * wcsconvertor() const;
  
  unsigned  xbins() const;

  /* The total number of elements in map */
  unsigned size()   const;

private:

  /* Disallow default constructor */
  TablePixelIterator();

  bool _firstcalled;

  unsigned long _index;

  unsigned long _xindexsize;
  unsigned long _yindexsize;

  unsigned long _tableelements;

  DataSet    * _set;

  Table      * _table;

  Column     * _xColumn;
  Column     * _yColumn;
  Column     * _valueColumn;
  
  ColumnData * _xColData;
  ColumnData * _yColData;
  ColumnData * _valueColData;
  
  int16      * _xData;
  int16      * _yData;
  real64     * _valueData;

  WCSConvertor * _wcsconv;

};

#endif
